@@ -22,11 +22,22 @@
 #
 # Created:
 #-------------------------------------------------------------------------------
+from .filesystem import *
+from .module_geolocate import *
 from .module_GDAL2Numpy import *
 from .module_Numpy2GTiff import *
 from .module_MetaData import *
 from .module_ogr import *
 from .module_features import *
+from .module_esri_shape import *
+from .module_geojson import ShapeFileFromGeoJSON
+from .module_s3 import *
+from .module_http import *
+from .module_open import get
 from .rasterlike import RasterLike
 from .rasterizelike import RasterizeLike
+from .module_gdal import *
 from .gdalwarp import gdalwarp
+from .module_log import *
+from .module_xml import *
+
