@@ -27,7 +27,9 @@ import datetime
 import json
 import os
 import tempfile
-
+import hashlib
+import shutil
+from .module_log import Logger
 
 def now():
     """
@@ -52,7 +54,9 @@ def normpath(pathname):
     """
     if not pathname:
         return ""
-    return os.path.normpath(pathname.replace("\\", "/")).replace("\\", "/")
+    pathname = os.path.normpath(pathname.replace("\\", "/")).replace("\\", "/")
+    pathname = pathname.replace(":/", "://") #patch for s3:// and http:// https://
+    return pathname
 
 
 def juststem(pathname):
@@ -60,7 +64,7 @@ def juststem(pathname):
     juststem
     """
     pathname = os.path.basename(pathname)
-    (root, ext) = os.path.splitext(pathname)
+    root, _ = os.path.splitext(pathname)
     return root
 
 
@@ -68,8 +72,8 @@ def justpath(pathname, n=1):
     """
     justpath
     """
-    for j in range(n):
-        (pathname, _) = os.path.split(normpath(pathname))
+    for _ in range(n):
+        pathname, _ = os.path.split(normpath(pathname))
     if pathname == "":
         return "."
     return normpath(pathname)
@@ -87,7 +91,7 @@ def justext(pathname):
     justext
     """
     pathname = os.path.basename(normpath(pathname))
-    (_, ext) = os.path.splitext(pathname)
+    _, ext = os.path.splitext(pathname)
     return ext.lstrip(".")
 
 
@@ -95,7 +99,7 @@ def forceext(pathname, newext):
     """
     forceext
     """
-    (root, _) = os.path.splitext(normpath(pathname))
+    root, _ = os.path.splitext(normpath(pathname))
     pathname = root + ("." + newext if len(newext.strip()) > 0 else "")
     return normpath(pathname)
 
@@ -114,6 +118,27 @@ def isshape(pathname):
     return pathname and os.path.isfile(pathname) and justext(pathname).lower() in ("shp",)
 
 
+def filesize(filename):
+    """
+    filesize
+    """
+    if os.path.isfile(filename):
+        return os.path.getsize(filename)
+    else:
+        return -1
+
+
+def filectime(filename):
+    """
+    filectime - get the creation date
+    """
+    if os.path.exists(filename):
+        unixtimestamp = os.path.getctime(filename)
+        return datetime.datetime.fromtimestamp(unixtimestamp).strftime("%Y-%m-%d %H:%M:%S")
+    else:
+        return None
+
+
 def mkdirs(pathname):
     """
     mkdirs - create a folder
@@ -122,16 +147,44 @@ def mkdirs(pathname):
         if os.path.isfile(pathname):
             pathname = justpath(pathname)
         os.makedirs(pathname, exist_ok=True)
-    except:
-        pass
+    except OSError as ex:
+        Logger.error(ex)
     return os.path.isdir(pathname)
 
+def remove(pathname):
+    """
+    remove - remove a file
+    """
+    if os.path.isfile(pathname):
+        try:
+            os.remove(pathname)
+        except OSError as ex:
+            Logger.error(ex)
+    elif os.path.isdir(pathname):
+        try:
+            shutil.rmtree(pathname)
+        except OSError as ex:
+            Logger.error(ex)
+    elif isinstance(pathname, (list, tuple)):
+        for filename in pathname:
+            remove(filename)
+
+
+def tempdir(name=""):
+    """
+    tempdir
+    :return: a temporary directory
+    """
+    foldername = normpath(tempfile.gettempdir() + "/" + name)
+    os.makedirs(foldername, exist_ok=True)
+    return foldername
+
 
 def tempfilename(prefix="", suffix=""):
     """
     return a temporary filename
     """
-    return tempfile.gettempdir() + "/" + datetime.datetime.strftime(now(), f"{prefix}%Y%m%d%H%M%S%f{suffix}")
+    return normpath(tempfile.gettempdir() + "/" + now().strftime(f"{prefix}%Y%m%d%H%M%S%f{suffix}"))
 
 
 def strtofile(text, filename, append=False):
@@ -148,8 +201,8 @@ def strtofile(text, filename, append=False):
         with open(filename, flag) as stream:
             if text:
                 stream.write(text)
-    except Exception as ex:
-        print(ex)
+    except OSError as ex:
+        Logger.error(ex)
         return ""
     return filename
 
@@ -168,7 +221,8 @@ def filetostr(filename):
     try:
         with open(filename, "r", encoding="utf-8") as stream:
             return stream.read()
-    except:
+    except OSError as ex:
+        Logger.error(ex)
         return None
 
 
@@ -179,7 +233,55 @@ def filetojson(filename):
     try:
         with open(filename, "r", encoding="utf-8") as stream:
             return json.load(stream)
-    except:
+    except OSError as ex:
+        Logger.error(ex)
         return None
 
+    
+def listify(text, sep=",", trim=False):
+    """
+    listify -  make a list from string
+    """
+    if text is None:
+        return []
+    elif isinstance(text, str):
+        arr = text.split(sep)
+        if trim:
+            arr = [item.strip() for item in arr]
+        return arr
+    elif isinstance(text, (tuple, list)):
+        return text
+    return [text]
 
+
+def md5text(text):
+    """
+    md5text - Returns the md5 of the text
+    """
+    if text is not None:
+        hashcode = hashlib.md5()
+        if isinstance(text, (bytes, bytearray)):
+            hashcode.update(text)
+        else:
+            hashcode.update(text.encode("utf-8"))
+        return hashcode.hexdigest()
+    return None
+
+
+def md5sum(filename):
+    """
+    md5sum - returns themd5 of the file
+    """
+    if os.path.isfile(filename):
+        f = open(filename, mode='rb')
+        d = hashlib.md5()
+        while True:
+            buf = f.read(4096)
+            if not buf:
+                break
+            d.update(buf)
+        f.close()
+        res=  d.hexdigest()
+        return res
+    else:
+        return ""
\ No newline at end of file
