@@ -24,8 +24,13 @@
 # -------------------------------------------------------------------------------
 import math
 import numpy as np
-from osgeo import gdal, gdalconst
+from osgeo import gdal
 from .filesystem import now, total_seconds_from, justfname
+from .module_s3 import *
+from .module_open import OpenRaster
+from .module_log import Logger
+from .module_memory import mem_usage
+
 
 def GDAL2Numpy(filename, band=1, dtype=np.float32, load_nodata_as=np.nan, bbox=[], verbose=False):
     """
@@ -35,36 +40,35 @@ def GDAL2Numpy(filename, band=1, dtype=np.float32, load_nodata_as=np.nan, bbox=[
     data_type_of = {
         'Float32': np.float32,
         'Float64': np.float64,
-        'CFloat32': np.float32,
-        'CFloat64': np.float64,
         'Byte': np.uint8,
         'Int16': np.int16,
         'Int32': np.int32,
         'UInt16': np.uint16,
         'UInt32': np.uint32,
-        'CInt16': np.int16,
-        'CInt32': np.int32
     }
 
-    filename = "/vsicurl/" + filename if filename and filename.lower().startswith("http") else filename
-    ds = gdal.Open(filename, gdalconst.GA_ReadOnly)
+    ds = OpenRaster(filename)
     if ds:
         band = ds.GetRasterBand(band)
         m, n = ds.RasterYSize, ds.RasterXSize
         gt, prj = ds.GetGeoTransform(), ds.GetProjection()
         no_data = band.GetNoDataValue()
         band_type = data_type_of[gdal.GetDataTypeName(band.DataType)]
+        Logger.debug("Data type: %s" % band_type)
 
         if not bbox:
             data = band.ReadAsArray(0, 0, n, m)
         else:
             x0, px, r0, y0, r1, py = gt
             X0, Y0, X1, Y1 = bbox
-
+            print("bbox", bbox)
             # calcutate starting indices
             j0, i0 = int((X0 - x0) / px), int((Y1 - y0) / py)
             cols, rows = math.ceil((X1 - X0) / px), math.ceil(abs(Y1 - Y0) / abs(py))
 
+            print("mxn", m,"x", n)
+            print("cols, rows", cols, "x", rows)    
+
             # index-safe
             j0, i0 = min(max(j0, 0), n - 1), min(max(i0, 0), m - 1)
             cols = min(max(cols, 0), n)
@@ -75,6 +79,8 @@ def GDAL2Numpy(filename, band=1, dtype=np.float32, load_nodata_as=np.nan, bbox=[
             h = math.floor((Y1 - y0) / py)
             gt = x0 + k * px, px, r0, y0 + h * py, r1, py
 
+            print("ReadAsArray(%d,%d,%d,%d)" % (j0, i0, cols, rows))
+
             data = band.ReadAsArray(j0, i0, cols, rows)
 
         # translate no-data as Nan
@@ -85,6 +91,7 @@ def GDAL2Numpy(filename, band=1, dtype=np.float32, load_nodata_as=np.nan, bbox=[
 
             # Output datatype
             if dtype and dtype != band_type:
+                Logger.debug("Converting data type from %s to %s" % (band_type, dtype))
                 data = data.astype(dtype, copy=False)
 
             if band_type == np.float32:
@@ -107,8 +114,8 @@ def GDAL2Numpy(filename, band=1, dtype=np.float32, load_nodata_as=np.nan, bbox=[
 
         band = None
         ds = None
-        if verbose:
-            print("Reading <%s> in %ss." % (justfname(filename), total_seconds_from(t0)))
+        mem_usage()
+        Logger.debug(f"Reading {justfname(filename)} in {total_seconds_from(t0)}s.")
         return data, gt, prj
-    print("file %s not exists!" % filename)
+    Logger.error(f"file <{filename}> not exists!")
     return None, None, None
