@@ -26,32 +26,46 @@ import os
 import numpy as np
 from osgeo import gdal, gdalconst
 from .filesystem import forceext, israster, isshape, filetojson, jsontofile
+from .module_s3 import isfile
 from .module_GDAL2Numpy import GDAL2Numpy
 from .module_Numpy2GTiff import Numpy2GTiff
 from .module_features import GetRange
+from .module_ogr import GetExtent
+from .module_open import OpenRaster
 
 
 def GetRasterShape(filename):
     """
     GetRasterShape
     """
-    ds = gdal.Open(filename, gdalconst.GA_ReadOnly)
+    ds = OpenRaster(filename)
     if ds:
         m, n = ds.RasterYSize, ds.RasterXSize
         return m, n
     return 0, 0
 
 
-def GetNoData(filename):
+def GetTransform(filename):
+    """
+    GetTransform
+    """
+    ds = OpenRaster(filename)
+    if ds:
+        gt = ds.GetGeoTransform()
+        return gt
+    return None
+
+
+def GetNoData(filename, band=1):
     """
     GetNoData
     """
-    ds = gdal.Open(filename, gdalconst.GA_ReadOnly)
+    ds = OpenRaster(filename)
     if ds:
-        band = ds.GetRasterBand(1)
-        no_data = band.GetNoDataValue()
-        data, band, ds = None, None, None
-        return no_data
+        # check if band exists
+        if band>0 and band <= ds.RasterCount:
+            bandx = ds.GetRasterBand(band)
+            return bandx.GetNoDataValue()
     return None
 
 
@@ -59,11 +73,11 @@ def SetNoData(filename, nodata):
     """
     SetNoData
     """
-    dataset = gdal.Open(filename, gdalconst.GA_Update)
-    if dataset:
-        band = dataset.GetRasterBand(1)
+    ds = OpenRaster(filename, gdalconst.GA_Update)
+    if ds:
+        band = ds.GetRasterBand(1)
         nodata = band.SetNoDataValue(nodata)
-        data, band, dataset = None, None, None
+        data, band, ds = None, None, None
     return None
 
 
@@ -71,7 +85,7 @@ def GDALFixNoData(filename, format="GTiff", nodata=-9999):
     """
     GDALFixNoData
     """
-    if os.path.isfile(filename):
+    if isfile(filename):
         data, gt, prj = GDAL2Numpy(filename, load_nodata_as=nodata)
         data[abs(data) >= 1e10] = nodata
         Numpy2GTiff(data, gt, prj, filename, format=format, save_nodata_as=nodata)
@@ -79,6 +93,23 @@ def GDALFixNoData(filename, format="GTiff", nodata=-9999):
     return False
 
 
+def IsEmpty(filename, nodata=-9999):
+    """
+    IsEmpty - check all values are nodata
+    """
+    if isfile(filename):
+        ds = OpenRaster(filename)
+        if ds:
+            band = ds.GetRasterBand(1)
+            nodata = band.GetNoDataValue()
+            data = band.ReadAsArray()
+            data[np.isnan(data)] = nodata
+            data[abs(data) >= 1e10] = nodata
+            ds = None
+            return np.all(data == nodata)
+    return False 
+
+
 def GetMinMax(filename, fieldname=None):
     """
     GetMinMax
@@ -91,39 +122,41 @@ def GetMinMax(filename, fieldname=None):
 
     return np.Inf, -np.Inf
 
+
 def GetMetaData(filename):
     """
     GetMetaData - get metadata from filename
     :param filename: the pathname
     :return: returns a dictionary with metadata
     """
-    if israster(filename):
-        ds = gdal.Open(filename, gdalconst.GA_ReadOnly)
-        if ds:
-            m, n = ds.RasterYSize, ds.RasterXSize
-            band = ds.GetRasterBand(1)
-            gt = ds.GetGeoTransform()
-            wkt = ds.GetProjection()
-            meta = ds.GetMetadata()
-            nodata = band.GetNoDataValue()
-            minx, px, _, maxy, _, py = gt
-            maxx = minx + n * px
-            miny = maxy + m * py
-            miny, maxy = min(miny, maxy), max(miny, maxy)
-            ds = None
-            return {
-                "m": m,
-                "n": n,
-                "px": px,
-                "py": py,
-                "wkt": wkt,
-                "nodata": nodata,
-                "extent": [minx, miny, maxx, maxy],
-                "metadata": meta
-            }
-    elif isshape(filename):
-        filemeta = forceext(filename, "mta")
-        return filetojson(filemeta)
+    if isfile(filename):
+        if filename.endswith(".tif"):
+            ds = OpenRaster(filename)
+            if ds:
+                m, n = ds.RasterYSize, ds.RasterXSize
+                band = ds.GetRasterBand(1)
+                gt = ds.GetGeoTransform()
+                wkt = ds.GetProjection()
+                meta = ds.GetMetadata()
+                nodata = band.GetNoDataValue()
+                minx, px, _, maxy, _, py = gt
+                maxx = minx + n * px
+                miny = maxy + m * py
+                miny, maxy = min(miny, maxy), max(miny, maxy)
+                ds = None
+                return {
+                    "m": m,
+                    "n": n,
+                    "px": px,
+                    "py": py,
+                    "wkt": wkt,
+                    "nodata": nodata,
+                    "extent": [minx, miny, maxx, maxy],
+                    "metadata": meta
+                }
+        elif filename.endswith(".shp"):
+            filemeta = forceext(filename, "mta")
+            return filetojson(filemeta)
 
     return {}
 
@@ -132,24 +165,25 @@ def GetTag(filename, tagname, band=0):
     """
     GetTag - get a tag in metadata of the file or of the band if specified
     """
-    if israster(filename):
-        ds = gdal.Open(filename, gdalconst.GA_ReadOnly)
-        if ds:
-            if not band:
-                metadata = ds.GetMetadata()
-            elif 0 < band <= ds.RasterCount:
-                metadata = ds.GetRasterBand(band).GetMetadata()
-            else:
-                metadata = {}
-            if tagname in metadata:
+    if isfile(filename):
+        if filename.endswith(".tif"):
+            ds = OpenRaster(filename)
+            if ds:
+                if not band:
+                    metadata = ds.GetMetadata()
+                elif 0 < band <= ds.RasterCount:
+                    metadata = ds.GetRasterBand(band).GetMetadata()
+                else:
+                    metadata = {}
+                if tagname in metadata:
+                    ds = None
+                    return metadata[tagname]
                 ds = None
-                return metadata[tagname]
-            ds = None
-    elif isshape(filename):
-        filemeta = forceext(filename, "mta")
-        meta = filetojson(filemeta)
-        if meta and "metadata" in meta and tagname in meta["metadata"]:
-            return meta["metadata"][tagname]
+        elif filename.endswith(".shp"):
+            filemeta = forceext(filename, "mta")
+            meta = filetojson(filemeta)
+            if meta and "metadata" in meta and tagname in meta["metadata"]:
+                return meta["metadata"][tagname]
 
     return None
 
@@ -159,7 +193,7 @@ def SetTag(filename, tagname, tagvalue="", band=0):
     SetTag - set a tag in metadata of the file or of the band if specified
     """
     if israster(filename):
-        ds = gdal.Open(filename, gdalconst.GA_Update)
+        ds = OpenRaster(filename, gdalconst.GA_Update)
         if ds:
             if tagname:
                 if not band:
@@ -175,8 +209,9 @@ def SetTag(filename, tagname, tagvalue="", band=0):
 
     elif isshape(filename):
         filemeta = forceext(filename, "mta")
-        meta = filetojson(filemeta)
-        meta = meta if meta else {"metadata": {}}
+        meta = {"metadata": {}}
+        if os.path.isfile(filemeta):
+            meta = filetojson(filemeta)
         if "metadata" in meta:
             meta["metadata"][tagname] = tagvalue
             jsontofile(meta, filemeta)
@@ -187,7 +222,7 @@ def SetTags(filename, meta, band=0):
     SetTags - set tags metadata of the file or of the band if specified
     """
     if israster(filename):
-        ds = gdal.Open(filename, gdalconst.GA_Update)
+        ds = OpenRaster(filename, gdalconst.GA_Update)
         if ds:
             for tagname in meta:
                 tagvalue = meta[tagname]
@@ -204,12 +239,31 @@ def SetTags(filename, meta, band=0):
 
     elif isshape(filename):
         filemeta = forceext(filename, "mta")
-        mta = filetojson(filemeta)
-        if not mta:
-            mta = {"metadata": {}}
-        if "metadata" in meta:
-            # update mta from meta
-            for tagname in meta["metadata"]:
-                tagvalue = meta["metadata"][tagname]
+        mta = {"metadata": {}}
+        if os.path.isfile(filemeta):
+            mta = filetojson(filemeta)
+        # update mta from meta
+        for tagname in meta:
+            tagvalue = meta[tagname]
+            if "metadata" in mta:
                 mta["metadata"][tagname] = tagvalue
-            jsontofile(mta, filemeta)
+        jsontofile(mta, filemeta)
+
+
+def setExtent(fileshp):
+    """
+    setExtent - set extent of a shapefile
+    """
+    if isshape(fileshp):
+        extent = GetExtent(fileshp)
+        filemeta = forceext(fileshp, "mta")
+        mta = filetojson(filemeta)
+        mta = {"metadata": {}} if not mta else mta
+        if "metadata" in mta:
+            mta["metadata"]["extent"] = {
+                "minx": extent[0],
+                "miny": extent[1],
+                "maxx": extent[2],
+                "maxy": extent[3]
+            }
+        jsontofile(mta, filemeta)
\ No newline at end of file
