@@ -25,11 +25,21 @@
 import glob
 import math
 import os
+import re
+import json
+import pkgutil
 import shutil
 import site
-from osgeo import gdal, gdalconst
+from osgeo import gdal
 from osgeo import osr, ogr
-from .filesystem import justext, juststem, forceext, justpath, isshape, israster, strtofile
+from pyproj import CRS
+from .filesystem import justext, juststem, forceext, justpath, strtofile, filetostr, md5text
+from .module_open import OpenRaster
+from .module_open import OpenShape
+from .module_s3 import isfile
+from .module_log import Logger
+from Levenshtein import distance
+
 
 def create_cpg(fileshp):
     """
@@ -39,6 +49,7 @@ def create_cpg(fileshp):
     """
     strtofile("UFT-8", forceext(fileshp, "cpg"))
 
+
 def ogr_move(src, dst):
     """
     copyshp
@@ -104,26 +115,28 @@ def Haversine(lat1, lon1, lat2, lon2):
     dLon = math.radians(lon2 - lon1)
     lat1 = math.radians(lat1)
     lat2 = math.radians(lat2)
-    a = math.sin(dLat / 2) ** 2 + math.cos(lat1) * math.cos(lat2) * math.sin(dLon / 2) ** 2
+    a = math.sin(dLat / 2) ** 2 + math.cos(lat1) * \
+        math.cos(lat2) * math.sin(dLon / 2) ** 2
     c = 2 * math.asin(math.sqrt(a))
     return R * c
 
 
-def GetPixelSize(filename):
+def GetPixelSize(filename, um="m"):
     """
     GetPixelSize
     """
-    ds = gdal.Open(filename, gdalconst.GA_ReadOnly)
+    ds = OpenRaster(filename)
     if ds:
         m, n = ds.RasterYSize, ds.RasterXSize
         minx, px, _, maxy, _, py = ds.GetGeoTransform()
         prj = ds.GetProjection()
         ds = None
 
-        srs = osr.SpatialReference()
-        srs.ImportFromProj4(prj)
+        # srs = osr.SpatialReference()
+        # srs.ImportFromProj4(prj)
+        srs = GetSpatialRef(prj)
 
-        if srs.IsGeographic():
+        if srs.IsGeographic() and um == "m":
             dx = Haversine(maxy, minx, maxy, minx + px * n) / n
             dy = Haversine(maxy, minx, maxy + m * py, minx) / m
             return round(dx, 1), round(dy, 1)
@@ -133,6 +146,14 @@ def GetPixelSize(filename):
     return None, None
 
 
+def GetPixelArea(filename, um="m"):
+    """
+    GetPixelArea
+    """
+    px, py = GetPixelSize(filename, um)
+    return abs(px * py)
+
+
 def SamePixelSize(filename1, filename2, decimals=-1):
     """
     SamePixelSize
@@ -145,6 +166,104 @@ def SamePixelSize(filename1, filename2, decimals=-1):
     return size1 == size2
 
 
+def AutoIdentify(wkt):
+    """
+    AutoIdentify
+    """
+    # get the file pe_hash_list.json from package data
+    code = None
+
+    if isfile(wkt) and wkt.endswith(".tif"):
+        wkt = OpenRaster(wkt).GetProjection()
+    elif isfile(wkt) and isfile(forceext(wkt, "prj")):
+        wkt = filetostr(forceext(wkt, "prj"))
+    elif isinstance(wkt, int):
+        wkt = f"EPSG:{wkt}"
+        return wkt
+    elif isinstance(wkt, osr.SpatialReference):
+        wkt = wkt.ExportToWkt()
+    elif isinstance(wkt, ogr.DataSource):
+        layer = wkt.GetLayer()
+        wkt = layer.GetSpatialRef().ExportToWkt()
+    elif isinstance(wkt, ogr.Layer):
+        wkt = wkt.GetSpatialRef().ExportToWkt()
+    elif isinstance(wkt, ogr.Feature):
+        wkt = wkt.GetGeometryRef().GetSpatialReference().ExportToWkt()
+    elif isinstance(wkt, ogr.Geometry):
+        wkt = wkt.GetSpatialReference().ExportToWkt()
+    elif isinstance(wkt, str) and re.match(r'^.*?\:\d{4,5}$', wkt):
+        return wkt
+    elif isinstance(wkt, str) and (wkt.startswith("GEOGCS") or wkt.startswith("PROJCS")):
+        pass
+    else:
+        Logger.warning("The wkt is not a valid string or object")
+        return None
+
+    if not code:
+        Logger.debug("1a) First chance to identify the wkt")
+        srs = CRS.from_wkt(wkt)
+        code = srs.to_epsg()
+        code = f"EPSG:{code}" if code else None
+
+    if not code:
+        Logger.debug("1) First chance to identify the wkt")
+        pe_hash_list = json.loads(pkgutil.get_data(
+            __name__, "data/pe_hash_list.json").decode("utf-8"))
+        pe_hash_list = pe_hash_list["CoordinateSystems"] if "CoordinateSystems" in pe_hash_list else {
+        }
+        code = pe_hash_list.get(md5text(wkt), None)
+
+    # last chance to identify the wkt
+    if not code:
+        Logger.debug("2) Second chance to identify the wkt")
+        spatial_res_sys_all = json.loads(pkgutil.get_data(
+            __name__, "data/spatial_ref_sys_all.json").decode("utf-8"))
+        spatial_res_sys_all = spatial_res_sys_all["CoordinateSystems"] if "CoordinateSystems" in spatial_res_sys_all else {
+        }
+        for authid, srs in spatial_res_sys_all.items():
+            if srs["wkt"] == wkt or srs["proj4"] == wkt:
+                code = authid
+                break
+
+    if not code:
+        Logger.debug("3) Third chance to identify the wkt by the name")
+        candidates = []
+        for authid, srs in spatial_res_sys_all.items():
+            if srs["name"] == osr.SpatialReference(wkt).GetName():
+                candidates.append((authid, distance(srs["wkt"], wkt)))
+
+        #  sort by distance
+        candidates = sorted(candidates, key=lambda x: x[1])
+        code = candidates[0][0] if len(candidates) > 1 else None
+
+    return code
+
+
+def isEPSG(epsg):
+    """
+    isEPSG - check if the string is a epsg
+    """
+    return isinstance(epsg, str) and epsg.lower().startswith("epsg")
+
+
+def isProj4(proj4):
+    """
+    isProj4 - check if the string is a proj4
+    """
+    return isinstance(proj4, str) and proj4.lower().startswith("+proj")
+
+
+def isWkt(wkt):
+    """
+    isWkt - check if the string is a wkt
+    """
+    if isinstance(wkt, str):
+        for word in ("GEOGCS", "PROJCS", "COMPD_CS"):
+            if wkt.upper().startswith(word):
+                return True
+    return False
+
+
 def GetSpatialRef(filename):
     """
     GetSpatialRef
@@ -156,35 +275,44 @@ def GetSpatialRef(filename):
     elif isinstance(filename, int):
         srs = osr.SpatialReference()
         srs.ImportFromEPSG(filename)
+        srs.AutoIdentifyEPSG()
 
-    elif isinstance(filename, str) and filename.lower().startswith("epsg:"):
+    elif isEPSG(filename):
         code = int(filename.split(":")[1])
         srs = osr.SpatialReference()
         srs.ImportFromEPSG(code)
+        srs.AutoIdentifyEPSG()
 
-    elif isinstance(filename, str) and filename.upper().startswith("+proj"):
+    elif isProj4(filename):
         proj4text = filename
         srs = osr.SpatialReference()
         srs.ImportFromProj4(proj4text)
+        srs.AutoIdentifyEPSG()
 
-    elif isinstance(filename, str) and filename.upper().startswith("PROJCS["):
+    elif isWkt(filename):
         wkt = filename
+        code = AutoIdentify(wkt)
         srs = osr.SpatialReference()
-        srs.ImportFromWkt(wkt)
+        if code:
+            code = int(code.split(":")[1])
+            srs.ImportFromEPSG(code)
+        else:
+            srs.ImportFromWkt(wkt)
+            srs.AutoIdentifyEPSG()
 
-    elif isinstance(filename, str) and isshape(filename):
-        ds = ogr.OpenShared(filename)
+    elif isinstance(filename, str) and isfile(filename) and filename.lower().endswith(".shp"):
+        ds = OpenShape(filename)
         if ds:
             srs = ds.GetLayer().GetSpatialRef()
-        ds = None
+            srs.AutoIdentifyEPSG()
 
-    elif isinstance(filename, str) and israster(filename):
-        ds = gdal.Open(filename, gdalconst.GA_ReadOnly)
+    elif isinstance(filename, str) and isfile(filename) and filename.lower().endswith(".tif"):
+        ds = OpenRaster(filename)
         if ds:
             wkt = ds.GetProjection()
             srs = osr.SpatialReference()
             srs.ImportFromWkt(wkt)
-        ds = None
+            srs.AutoIdentifyEPSG()
     else:
         srs = osr.SpatialReference()
 
@@ -198,7 +326,7 @@ def SameSpatialRef(filename1, filename2):
     srs1 = GetSpatialRef(filename1)
     srs2 = GetSpatialRef(filename2)
     if srs1 and srs2:
-        return srs1.IsSame(srs2)
+        return srs1.IsSame(srs2) or srs1.ExportToProj4() == srs2.ExportToProj4()
     return None
 
 
@@ -208,7 +336,7 @@ def GetGeometryType(filename):
     :param filename:
     :return:
     """
-    ds = ogr.OpenShared(filename)
+    ds = OpenShape(filename, 0)
     if ds:
         lyr = ds.GetLayer()
         if lyr:
@@ -219,6 +347,7 @@ def GetGeometryType(filename):
         ds = None
     return None
 
+
 def Rectangle(minx, miny, maxx, maxy):
     """
     Rectangle - create ogr polygon from bbox
@@ -235,15 +364,42 @@ def Rectangle(minx, miny, maxx, maxy):
     return poly
 
 
+def TransformBBOX(bbox, s_srs=None, t_srs=None):
+    """
+    TransformBBOX
+    """
+    if SameSpatialRef(s_srs, t_srs):
+        return bbox
+    s_minx, s_miny, s_maxx, s_maxy = bbox
+
+    if s_srs and s_srs.IsGeographic():
+        s_srs.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
+    if t_srs and t_srs.IsGeographic():
+        t_srs.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
+
+    transform = osr.CoordinateTransformation(s_srs, t_srs)
+    # rect = Rectangle(s_minx, s_miny, s_maxx, s_maxy)
+    # rect.Transform(transform)
+    # t_minx, t_maxx, t_miny, t_maxy = rect.GetEnvelope()
+    # transformed_bbox = (t_minx, t_miny, t_maxx, t_maxy)
+    transformed_bbox = transform.TransformBounds(s_minx, s_miny, s_maxx, s_maxy, 0)
+    return transformed_bbox
+
+
 def GetExtent(filename, t_srs=None):
     """
     GetExtent
     """
     s_srs = None
     minx, miny, maxx, maxy = 0, 0, 0, 0
-    ext = justext(filename).lower()
-    if ext == "tif":
-        ds = gdal.Open(filename, gdalconst.GA_ReadOnly)
+    ext = justext(f"{filename}").lower()
+    if isinstance(filename, (list, tuple)):
+        minx, miny, maxx, maxy = filename
+        s_srs = GetSpatialRef(4326)
+        s_srs.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
+        
+    elif ext == "tif":
+        ds = OpenRaster(filename)
         if ds:
             "{xmin} {ymin} {xmax} {ymax}"
             m, n = ds.RasterYSize, ds.RasterXSize
@@ -260,8 +416,9 @@ def GetExtent(filename, t_srs=None):
     elif ext in ("shp", "dbf"):
 
         filename = forceext(filename, "shp")
-        driver = ogr.GetDriverByName("ESRI Shapefile")
-        ds = driver.Open(filename, 0)
+        # driver = ogr.GetDriverByName("ESRI Shapefile")
+        # ds = driver.Open(filename, 0)
+        ds = OpenShape(filename, 0)
         if ds:
             layer = ds.GetLayer()
             minx, maxx, miny, maxy = layer.GetExtent()
@@ -269,15 +426,10 @@ def GetExtent(filename, t_srs=None):
             ds = None
 
     if t_srs and not SameSpatialRef(s_srs, t_srs):
-        transform = osr.CoordinateTransformation(s_srs, GetSpatialRef(t_srs))
-        rect = Rectangle(minx, miny, maxx, maxy)
-        rect.Transform(transform)
-        if f"{t_srs}" == "4326":
-            miny, maxy, minx, maxx = rect.GetEnvelope()
-        else:
-            minx, miny, maxx, maxy = rect.GetEnvelope()
-
-        miny, maxy = min(miny, maxy), max(miny, maxy)
+        s_srs = GetSpatialRef(s_srs)
+        t_srs = GetSpatialRef(t_srs)
+        transform = osr.CoordinateTransformation(s_srs, t_srs)
+        minx, miny, maxx, maxy = transform.TransformBounds(minx, miny, maxx, maxy, 0)
 
     return minx, miny, maxx, maxy
 
@@ -324,7 +476,7 @@ def find_PROJ_LIB():
         pathnames += glob.glob(root + "/**/proj.db", recursive=True)
         if len(pathnames):
             break
-    return justpath(pathnames[0]) if len(pathnames) else ""
+    return justpath(pathnames[0]) if len(pathnames)>0 else ""
 
 
 def find_GDAL_DATA():
@@ -337,23 +489,7 @@ def find_GDAL_DATA():
         pathnames += glob.glob(root + "/**/gt_datum.csv", recursive=True)
         if len(pathnames):
             break
-    return justpath(pathnames[0]) if len(pathnames) else ""
-
-
-def Rectangle(minx, miny, maxx, maxy):
-    """
-    Rectangle
-    """
-    ring = ogr.Geometry(ogr.wkbLinearRing)
-    ring.AddPoint_2D(minx, miny)
-    ring.AddPoint_2D(maxx, miny)
-    ring.AddPoint_2D(maxx, maxy)
-    ring.AddPoint_2D(minx, maxy)
-    ring.AddPoint_2D(minx, miny)
-    # Create polygon
-    poly = ogr.Geometry(ogr.wkbPolygon)
-    poly.AddGeometry(ring)
-    return poly
+    return justpath(pathnames[0]) if len(pathnames)>0 else ""
 
 
 def CreateRectangleShape(minx, miny, maxx, maxy, srs, fileshp="tempxy...."):
@@ -399,4 +535,4 @@ def CopyShape(fileshp, fileout):
     """
     ds = gdal.VectorTranslate(fileout, fileshp, format='ESRI Shapefile',
                               accessMode='overwrite')
-    ds = None  # force flush
\ No newline at end of file
+    ds = None  # force flush
